# Copyright (c) 2010 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QTimer

from dip.model import Int, List, Model

from .robot_command import RobotCommand


class Robot(Model):
    """ The Robot class manages the automation commands that can be executed by
    a user interface.
    """

    # The default delay in milliseconds between the playing of individual
    # automation commands.
    delay = Int(-1)

    # The default time in milliseconds to wait for a user interface widget to
    # be visible.
    timeout = Int(-1)

    # The list of commands that can be played.
    _commands = List(RobotCommand)

    def clear(self):
        """ Any recorded automation commands are discarded. """

        self._commands = []

    def play(self, after=-1):
        """ Any recorded automation commands are executed.

        :param after:
            is the delay in milliseconds after the start of an event loop
            before the commands are executed.  If the value is negative then
            the commands are played immediately without waiting for an event
            loop to start.
        """

        if after < 0:
            self._play_commands()
        else:
            QTimer.singleShot(after, self._play_commands)

    def _play_commands(self):
        """ Play the recorded commands. """

        for command in self._commands:
            command()

    def record(self, id, command, *command_args, delay=-1, timeout=-1):
        """ Record an automation command.

        :param id:
            is the (possibly scoped) identifier of the widget to apply the
            command to.
        :param command:
            is the name of the command to record.
        :param \*command_args:
            are the commands's arguments.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param timeout:
            is the time in milliseconds to wait for the user interface widget
            to become visible.
        """

        # Apply the defaults if needed.
        if delay < 0:
            delay = self.delay

        if timeout < 0:
            timeout = self.timeout

        command = RobotCommand(id=id, command=command,
                command_args=command_args, delay=delay, timeout=timeout)
        self._commands.append(command)

    @staticmethod
    def simulate(id, command, *command_args, delay=-1, timeout=-1):
        """ Immediately execute an automation command.

        :param id:
            is the (possibly scoped) identifier of the widget to apply the
            command to.
        :param command:
            is the name of the command to execute.
        :param \*command_args:
            are the command's arguments.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param timeout:
            is the time in milliseconds to wait for the user interface widget
            to become visible.
        """

        command = RobotCommand(id=id, command=command,
                command_args=command_args, delay=delay, timeout=timeout)
        command()
