# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Callable, Enum, List, Str
from ..ui import ViewFactory

from .i_shell import IShell


class BaseShellFactory(ViewFactory):
    """ The BaseShellFactory class is a base class for shell factories.  It
    creates views that can be adapted to the :class:`~dip.shell.IShell`
    interface.
    """

    # The identifier of a shell.
    id = 'dip.shell'

    # This determines how multiple views placed in the main area of the shell
    # are handled.  'multi' means that the area may contain any number of views
    # and will visualise the area in the same way irrespective of the number of
    # views.  'on_demand' means that the area may contain any number of views
    # but it may visualise the area differently if it contains a single view.
    # 'single' means that the area may only ever contain one view and an old
    # view must be discarded before a new view is added.
    main_area_policy = Enum('multi', 'on_demand', 'single')

    # The tool factories.  A factory should return an object that implements or
    # can be adapted to :class:`~dip.shell.ITool`.
    tool_factories = List(Callable())

    # The template from which the window title is derived.  Tokens in the
    # template are replaced by their corresponding values:
    #
    # [view] is replaced by the :attr:`~dip.ui.IDisplay.name` of the active
    # view.
    #
    # Note that, as with :attr:`~dip.ui.IView.window_title`, [*] will be
    # replaced with the application modification state by the toolkit.
    window_title_template = Str()

    def create(self, model, controller, parent):
        """ Create the shell view.

        :param model:
            is the model.
        :param controller:
            is the controller.
        :param parent:
            is the optional parent view.
        :return:
            the view which can be adapted to the :class:`~dip.shell.IShell`
            interface.
        """

        view = self.create_shell(model, controller, parent)

        shell = IShell(view)

        shell.main_area_policy = self.main_area_policy
        shell.window_title_template = self.window_title_template

        # Add any initial tools.
        shell.tools = [factory() for factory in self.tool_factories]

        self.configure(view)

        return view

    def create_shell(self, model, controller, parent):
        """ Create the view that implements the shell.  This should be
        reimplemented by a sub-class.

        :param model:
            is the model.
        :param controller:
            is the controller.
        :param parent:
            is the optional parent view.
        :return:
            the shell.
        """

        raise NotImplementedError
