# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ....model import Callable, implements, Instance, observe, Str, unadapted
from ....ui import IViewBinding, ViewFactory

from ... import IAreaHints, BaseManagedModelTool, IManagedModelTool


@implements(IManagedModelTool)
class FormTool(BaseManagedModelTool):
    """ The FormTool class is a :term:`tool` that uses an automatically
    generated :term:`view` (a :class:`~dip.ui.Form` by default) to manipulate a
    :term:`managed model`.
    """

    # The identifier of the area where the view should be placed.
    area = Str()

    # The controller factory.  This must return an object that implements or
    # can be adapted to the :class:`~dip.ui.IController` interface.
    controller_factory = Callable()

    # The type of model that the tool can handle.
    model_type = Instance(type)

    # The factory used to create the tool's views.
    view_factory = Instance(ViewFactory)

    def create_views(self, model):
        """ Create the views for a managed model.

        :param model:
            is the model.
        :return:
            the list of views.
        """

        controller = None if self.controller_factory is None else self.controller_factory()

        view = self.view_factory(unadapted(model), controller=controller,
                top_level=False)

        # Initialise the invalid reason.
        controller = IViewBinding(view).controller
        model.invalid_reason = controller.invalid_reason

        # Handle future changes.
        def update_invalid_reason(change):
            model.invalid_reason = change.new

        observe('invalid_reason', controller, update_invalid_reason)

        # Maintain the dirty state.
        def update_dirty(change):
            model.dirty = True

        observe('*', unadapted(model), update_dirty)

        # Position the view.
        IAreaHints(view).area = self.area

        return [view]

    def handles(self, model):
        """ Check if the tool can handle a particular model.

        :param model:
            is the model.
        :return:
            ``True`` if a tool can handle the model.
        """

        return isinstance(model, self.model_type)

    @view_factory.default
    def view_factory(self):
        """ Invoked to return the default view factory. """

        from ....ui import Form

        return Form()
