# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Interface


class IToolkit(Interface):
    """ The IToolkit interface defines the API to be implemented by a
    :term:`toolkit`.
    """

    def action(self, parent):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface.

        :param parent:
            is the optional parent view.
        :return:
            the action.
        """

    def application(self, argv, **properties):
        """ Create a singleton application, i.e. an object that implements the
        :class:`~dip.ui.IApplication` interface.

        :param argv:
            is the sequence of command line arguments.
        :param \*\*properties:
            are the keyword arguments used as toolkit specific property names
            and values that are used to configure the application.
        :return:
            the application.  Repeated calls must return the same application.
        """

    def check_box(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.ICheckBox`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def close_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle closing an object.

        :return:
            the action.
        """

    def combo_box(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IComboBox`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def declarative_factory_for_attribute_type(self, attribute_type):
        """ Get an :class:`~dip.ui.EditorFactory` sub-class that will create a
        factory that will create an editor to handle attributes of a particular
        type.

        :param attribute_type:
            is the type of the attribute.
        :return:
            the :class:`~dip.ui.EditorFactory` sub-class.
        """

    def editor_factory_for_attribute_type(self, attribute_type):
        """ Get a callable that will create a editor to handle attributes of a
        particular type.

        :param attribute_type:
            is the type of the attribute.
        :return:
            the callable.
        """

    def designer_load(self, ui_file_name):
        """ Load a file containing a user interface design.

        :param ui_file_name:
            is the name of the file.
        :return:
            an opaque object that will be passed to
            :meth:`~dip.toolkits.IToolkit.designer_view`.
        """

    def designer_view(self, design, parent):
        """ Create a view from a previously loaded user interface design.

        :param design:
            is an opaque object returned by a previous call to
            :meth:`~dip.toolkits.IToolkit.designer_load`.
        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def dialog(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IDialog`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def displayable(self, view):
        """ Return a view that can be displayed as a top-level window.

        :param view:
            is the view to be displayed.
        :return:
            the view, possibly wrapped so that it can be displayed.
        """

    def dock(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IDock` interface..

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def filesystem_location_editor(self, parent):
        """ Create an editor that implements the
        :class:`~dip.ui.IFilesystemLocationEditor` interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def find_toolkit_view(self, toolkit_root, id):
        """ Find the toolkit view with a particular identifier.

        :param toolkit_root:
            is the root toolkit view to begin the search.
        :param id:
            is the identifier of the view to search for.
        :return:
            the toolkit view.  An exception will be raised if the view could
            not be found.
        """

    def float_spin_box(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IFloatSpinBox`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def form(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IForm` interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def get_open_file(self, window_title, directory='', filter='', parent=None):
        """ Get the name of a file to open from the user.

        :param window_title:
            is the window title, typically used as the title of a dialog.
        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent view.
        :return:
            the name of the file to open or an empty string if there was none.
        """

    def get_save_file(self, window_title, directory='', filter='', parent=None):
        """ Get the name of a file to save from the user.

        :param window_title:
            is the window title, typically used as the title of a dialog.
        :param directory:
            is the name of an optional initial directory or file.
        :param filter:
            is the optional file filter.
        :param parent:
            is the optional parent view.
        :return:
            the name of the file to save or an empty string if there was none.
        """

    def grid(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IGrid` interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def group_box(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IGroupBox`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def h_box(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IHBox` interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def information(self, window_title, text, parent):
        """ Display a informational message to the user.

        :param window_title:
            is the window title, typically used as the title of a dialog.
        :param text:
            is the text of the message.
        :param parent:
            is the optional parent view.
        """

    def label(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.ILabel`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def line_editor(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.ILineEditor`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def list_editor(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IListEditor`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def main_window(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IMainWindow`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def menu(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IMenu` interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def menu_bar(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IMenuBar`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def message_area(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IMessageArea`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def new_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle creating a new object.

        :return:
            the action.
        """

    def open_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle opening an object.

        :return:
            the action.
        """

    def option_list(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IOptionList`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def push_button(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IPushButton`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def question(self, window_title, text, parent, detail, buttons):
        """ Ask the user a question.

        :param window_title:
            is the window title, typically used as the title of a dialog.
        :param text:
            is the text of the question.
        :param parent:
            is the optional parent view.
        :param detail:
            is the optional additional detail.
        :param buttons:
            is the sequence of buttons to display.  Possible buttons are 'yes',
            'no', 'cancel', 'save' and 'discard'.  The first in the sequence is
            used as the default.
        :return:
            The button that was pressed.
        """

    def quit_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle quitting the application.

        :return:
            the action.
        """

    def radio_buttons(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IRadioButtons`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def save_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle saving an object.

        :return:
            the action.
        """

    def save_as_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle saving an object under a
        new name.

        :return:
            the action.
        """

    def spin_box(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.ISpinBox`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def splitter(self, parent):
        """ Create a view that implements the :class:`~dip.ui.ISplitter`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def storage_location_editor(self, parent):
        """ Create an editor that implements the
        :class:`~dip.ui.IStorageLocationEditor` interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def tab_bar(self, parent):
        """ Create a view that implements the :class:`~dip.ui.ITabBar`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def tab_page(self, parent):
        """ Create a view that implements the :class:`~dip.ui.ITabPage`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def table_editor(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.ITableEditor`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def text_editor(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.ITextEditor`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def tool_button(self, parent):
        """ Create an editor that implements the :class:`~dip.ui.IToolButton`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the editor.
        """

    def v_box(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IVBox` interface.

        :param parent:
            is the optional parent view.
        :return:
            the layout.
        """

    def view_stack(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IViewStack`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the layout.
        """

    def warning(self, window_title, text, parent, detail):
        """ Display a warning message to the user.

        :param window_title:
            is the window title, typically used as the title of a dialog.
        :param text:
            is the text of the warning.
        :param parent:
            is the optional parent view.
        :param detail:
            is the optional additional detail.
        """

    def whats_this_action(self):
        """ Create an action, i.e. an object that implements the
        :class:`~dip.ui.IAction` interface, to handle "What's This?".

        :return:
            the action.
        """

    def wizard(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IWizard`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

    def wizard_page(self, parent):
        """ Create a view that implements the :class:`~dip.ui.IWizardPage`
        interface.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """
