# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QAbstractButton, QFileDialog, QLayout, QLineEdit

from ....model import adapt, Instance, Model
from ....ui import IFilesystemLocationEditor

from ..editor_layout_adapter import EditorLayoutAdapter
from ..visual_items import find_visual_item


class FilesystemLocationEditorComponents(Model):
    """ The FilesystemLocationEditorComponents class is an internal mixin for
    adapters that access the embedded components of a filesystem location
    editor.
    """

    # The line edit.
    line_edit = Instance(QLineEdit)

    @line_edit.default
    def line_edit(self):
        """ Invoked to return the tree widget. """

        return find_visual_item(self.adaptee, QLineEdit)

    # The browse button.
    browse_button = Instance(QAbstractButton)

    @browse_button.default
    def browse_button(self):
        """ Invoked to return the add button. """

        return find_visual_item(self.adaptee, QAbstractButton)

    @classmethod
    def usable_editor(cls, item):
        """ Return True if an item can be used as an editor. """

        # The line edit must be an immediate child otherwise the criteria would
        # be far too wide.
        if find_visual_item(item, QLineEdit, recurse=False) is None:
            return False

        if find_visual_item(item, QAbstractButton) is None:
            return False

        return True


@adapt(QLayout, to=IFilesystemLocationEditor)
class QLayoutIFilesystemLocationEditorAdapter(EditorLayoutAdapter, FilesystemLocationEditorComponents):
    """ An adapter to implement IFilesystemLocationEditor for a filesystem
    location editor that embeds a :class:`~PyQt4.QtGui.QLineEdit` and a
    :class:`~PyQt4.QtGui.QAbstractButton` that implements a browse button.
    """

    @classmethod
    def isadaptable(cls, adaptee):
        """ Determine if this adapter can adapt a particular layout.

        :param adaptee:
            is the layout.
        :return:
            ``True`` if the layout can be adapted.
        """

        return cls.usable_editor(adaptee)

    def configure(self, properties):
        """ Configure the editor. """

        line_edit = self.line_edit

        # For automation purposes give the line edit an the id if it doesn't
        # already have one.
        if line_edit.objectName() == '':
            line_edit.setObjectName(self.id)

        # Connect things up.
        line_edit.textEdited.connect(self.qt4_notify)
        self.browse_button.clicked.connect(self._on_clicked)

        # Apply any properties to the line edit.
        try:
            line_edit.pyqtConfigure(**properties)
        except AttributeError:
            pass

    @IFilesystemLocationEditor.read_only.getter
    def read_only(self):
        """ Invoked to get the read-only state. """

        return self.line_edit.isReadOnly()

    @read_only.setter
    def read_only(self, value):
        """ Invoked to set the read-only state. """

        self.line_edit.setReadOnly(value)
        self.browse_button.setVisible(not value)

    @IFilesystemLocationEditor.value.getter
    def value(self):
        """ Invoked to get the editor's value. """

        return self.line_edit.text()

    @value.setter
    def value(self, value):
        """ Invoked to set the editor's value. """

        # Save for the notifier.
        self.qt4_old_value = value

        self.line_edit.setText(value)

    def set_focus(self):
        """ Give the focus to the view. """

        self.line_edit.setFocus()

    def _on_clicked(self):
        """ Invoked when the browse button is clicked. """

        value = self.value
        mode = self.mode
        filter = self.filter
        line_edit = self.line_edit

        if mode == 'open_file':
            value = QFileDialog.getOpenFileName(line_edit, directory=value,
                    filter=filter)
            ok = (value != '')

        elif mode == 'save_file':
            value = QFileDialog.getSaveFileName(line_edit, directory=value,
                    filter=filter)
            ok = (value != '')

        elif mode == 'directory':
            value = QFileDialog.getExistingDirectory(line_edit,
                    directory=value)
            ok = (value != '')

        line_edit.activateWindow()
        line_edit.setFocus()

        if ok:
            self.value = value
