# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QPoint, Qt
from PyQt4.QtGui import QBoxLayout, QButtonGroup, QRadioButton
from PyQt4.QtTest import QTest

from ....automate import IAutomatedEditor, AutomationError
from ....model import adapt, Adapter, Dict, Instance, List, Model
from ....ui import IDisplay, IRadioButtons

from ..editor_layout_adapter import EditorLayoutAdapter
from ..visual_items import find_visual_item, visual_items
from ..widgets import DipRadioButtons


class RadioButtonsComponents(Model):
    """ The RadioButtonsComponents class is an internal mixin for adapters that
    access the embedded components of radio buttons.
    """

    # The optional button group.
    button_group = Instance(QButtonGroup)

    # The list of buttons.
    buttons = List(QRadioButton)

    @button_group.default
    def button_group(self):
        """ Invoked to return the button group. """

        layout = self.adaptee

        # This should succeed for DipRadioButtons.
        button_group = layout.findChild(QButtonGroup)

        if button_group is None:
            buttons = self.buttons
            if len(buttons) != 0:
                button = buttons[0]

                # Designer creates button groups as siblings of the buttons.
                # Allow for there being more than one button group.
                for button_group in button.parent().findChildren(QButtonGroup):
                    if button in button_group.buttons():
                        break
                else:
                    button_group = None

        return button_group

    @buttons.default
    def buttons(self):
        """ Invoked to return the list of buttons. """

        return [itm for itm in visual_items(self.adaptee, recurse=False)
                if isinstance(itm, QRadioButton)]

    @classmethod
    def usable_editor(cls, item):
        """ Return True if an item can be used as an editor. """

        if isinstance(item, DipRadioButtons):
            return True

        return find_visual_item(item, QRadioButton, recurse=False) is not None


@adapt(QBoxLayout, to=IRadioButtons)
class QBoxLayoutIRadioButtonsAdapter(EditorLayoutAdapter, RadioButtonsComponents):
    """ An adapter to implement IRadioButtons for a box layout that embeds at
    least one :class:`~PyQt4.QtGui.QRadioButton` (so it has been explicitly
    bound) or is a ``DipRadioButtons`` sub-class (and so will be populated with
    radio buttons as part of its configuration).
    """

    # The properties to configure new buttons with.
    _properties = Dict()

    @classmethod
    def isadaptable(cls, adaptee):
        """ Determine if this adapter can adapt a particular layout.

        :param adaptee:
            is the layout.
        :return:
            ``True`` if the layout can be adapted.
        """

        return cls.usable_editor(adaptee)

    def configure(self, properties):
        """ Configure the editor. """

        self._properties = properties

        for button in self.buttons:
            self._configure_button(button)

    @IRadioButtons.labels.setter
    def labels(self, value):
        """ Invoked to set the labels. """

        self._set_button_text(self.options, value)

    @IRadioButtons.options.setter
    def options(self, value):
        """ Invoked to set the options. """

        layout = self.adaptee
        button_group = self.button_group

        # Create or delete buttons as required to match the number of options.
        nr_buttons = len(self.buttons)
        nr_options = len(value)

        if nr_buttons > nr_options:
            for i in range(nr_options, nr_buttons):
                button = self.buttons.pop[nr_options]
                layout.removeWidget(button)

                if button_group is not None:
                    button_group.removeButton(button)

        elif nr_buttons < nr_options:
            for i in range(nr_buttons, nr_options):
                button = QRadioButton()
                self._configure_button(button)

                # Add to the layout allowing for it to have a stretch at the
                # end.
                layout.insertWidget(i, button)

                if button_group is not None:
                    button_group.addButton(button)

                self.buttons.append(button)

        # Set up the options in the buttons.
        for i in range(nr_options):
            self.buttons[i]._dip_option = value[i]

        self._set_button_text(value, self.labels)

    @IRadioButtons.read_only.setter
    def read_only(self, value):
        """ Invoked to set the read-only state. """

        for button in self.buttons:
            button.setEnabled(not value)

    @IRadioButtons.sorted.setter
    def sorted(self, value):
        """ Invoked to set the sorted state. """

        if value:
            self._sort()

    @IRadioButtons.value.getter
    def value(self):
        """ Invoked to get the editor's value. """

        for button in self.buttons:
            if button.isChecked():
                return button._dip_option

        return None

    @value.setter
    def value(self, value):
        """ Invoked to set the editor's value. """

        # Save for the notifier.
        self.qt4_old_value = value

        if value is None:
            for button in self.buttons:
                button.setChecked(False)
        else:
            for button in self.buttons:
                if button._dip_option == value:
                    button.setChecked(True)
                    break
            else:
                raise ValueError("invalid option: {0}".format(value))

    @IRadioButtons.vertical.getter
    def vertical(self):
        """ Invoked to get the orientation. """

        return self.adaptee.direction() in (
                QBoxLayout.TopToBottom, QBoxLayout.BottomToTop)

    @vertical.setter
    def vertical(self, value):
        """ Invoked to set the orientation. """

        self.adaptee.setDirection(
                QBoxLayout.TopToBottom if value else QBoxLayout.LeftToRight)

    def _sort(self):
        """ Sort the buttons according to their labels. """

        layout = self.adaptee

        # Remove all the buttons from the layout.
        for button in self.buttons:
            layout.removeWidget(button)

        # Sort the buttons.
        buttons = sorted(buttons, key=lambda button: button.text(),
                reverse=True)

        # Re-add them to the layout allowing for it to have a stretch at the
        # end.
        for button in buttons:
            layout.insertWidget(0, button)

    def _set_button_text(self, options, labels):
        """ Set the text of each button. """

        for button in self.buttons:
            option = button._dip_option
            idx = options.index(option)

            try:
                text = labels[idx]
            except IndexError:
                idisplay = IDisplay(option, exception=False)
                text = '' if idisplay is None else idisplay.name
                if text == '':
                    text = str(option)

            button.setText(text)

        if self.sorted:
            self._sort()

    def _configure_button(self, button):
        """ Configure a new button. """

        button.setEnabled(not self.read_only)

        try:
            button.pyqtConfigure(**self._properties)
        except AttributeError:
            pass

        button.clicked[()].connect(self._on_button_clicked)

    def _on_button_clicked(self):
        """ Invoked when a button is clicked. """

        self.qt4_notify(self.value)


@adapt(QBoxLayout, to=IAutomatedEditor)
class QBoxLayoutIAutomatedEditorAdapter(Adapter, RadioButtonsComponents):
    """ An adapter to implement IAutomatedEditor for a box layout that embeds
    at least one :class:`~PyQt4.QtGui.QRadioButton` (so it has been explicitly
    bound) or is a ``DipRadioButtons`` sub-class (and so will be populated with
    radio buttons as part of its configuration).
    """

    @classmethod
    def isadaptable(cls, adaptee):
        """ Determine if this adapter can adapt a particular layout.

        :param adaptee:
            is the layout.
        :return:
            ``True`` if the layout can be adapted.
        """

        return cls.usable_editor(adaptee)

    def simulate_set(self, id, delay, value):
        """ Simulate the user setting the editor value.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param value:
            is the value to set.  If the layout has implemented IOptionSelector
            then the value must be valid for the options.  Otherwise the value
            must be the integer index of the button in the layout.
        """

        # Try and find a button for the option.
        for button in self.buttons:
            try:
                if button._dip_option == value:
                    break
            except AttributeError:
                pass
        else:
            try:
                button = self.buttons[value]
            except (IndexError, ValueError):
                raise AutomationError(id, 'set',
                        "invalid value: {0}".format(value))

        QTest.mouseClick(button, Qt.LeftButton, Qt.NoModifier, QPoint(), delay)
