# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QEvent, QObject
from PyQt4.QtGui import QFormLayout, QLayout

from ....model import adapt, Bool, Instance, unadapted
from ....ui import IForm, IView

from ..layout_adapter import LayoutAdapter


@adapt(QFormLayout, to=IForm)
class QFormLayoutIFormAdapter(LayoutAdapter):
    """ An adapter to implement IForm for a QFormLayout. """

    # Set if configure() has been called.
    _qt4_configured = Bool(False)

    # The field visibility event filter.
    _qt4_visibility_event_filter = Instance(QObject)

    @IForm.views.setter
    def views(self, value):
        """ Invoked to set the form's views. """

        # Delay the initial setting of the views until we know the initial
        # value of the labels.
        if self._qt4_configured:
            self._qt4_set_views(value)

    def configure(self, properties):
        """ Configure the object using a mapping of toolkit specific property
        names and values.

        :param properties:
            is the mapping of the properties.
        """

        self._qt4_set_views(self._views)
        self._qt4_configured = True

        super().configure(properties)

    def _qt4_set_views(self, views):
        """ Set the views. """

        layout = self.adaptee

        # Clear the layout.
        while layout.count() > 0:
            layout.takeAt(0)

        # Populate the layout.
        for row, field in enumerate(views):
            field = unadapted(field)

            try:
                label = self.labels[row]
            except IndexError:
                label = None

            if label is None:
                if isinstance(field, QLayout):
                    layout.setLayout(row, QFormLayout.SpanningRole, field)
                else:
                    layout.setWidget(row, QFormLayout.SpanningRole, field)
            else:
                layout.addRow(label, field)

                # Set the label's enabled and visible states to match the
                # field's.  Note that Qt won't actually create a label widget
                # if the label is an empty string.
                label_w = layout.labelForField(field)
                if label_w is not None:
                    if isinstance(field, QLayout):
                        iview = IView(field)
                        enabled = iview.enabled
                        visible = iview.visible
                    else:
                        # Install an event filter for the field to monitor its
                        # visibility.
                        field.installEventFilter(
                                self._qt4_visibility_event_filter)

                        enabled = field.isEnabled()
                        visible = field.isVisible()

                    # Only change these if they are not the defaults, otherwise
                    # let the form do what it wants.
                    if not enabled:
                        label_w.setEnabled(False)

                    if not visible:
                        label_w.setVisible(False)

        self.qt4_configure(views)

    @_qt4_visibility_event_filter.default
    def _qt4_visibility_event_filter(self):
        """ Invoked to return the default field visibility event filter. """

        return VisibilityEventFilter(self.adaptee)


class VisibilityEventFilter(QObject):
    """ VisibilityEventFilter is an internal class that monitors the visibility
    of field widgets.
    """

    def eventFilter(self, obj, event):
        """ Reimplemented to handle any hide and show events. """

        event_type = event.type()

        if event_type == QEvent.Hide:
            self.parent().labelForField(obj).setVisible(False)
        elif event_type == QEvent.Show:
            label = self.parent().labelForField(obj)
            # FIXME: This doesn't seem to come into effect until the window is
            #        activated.
            label.setVisible(True)

            # We also make sure the label's enabled state is up to date.  We
            # can't rely on the setter in the widget adapter because the field
            # widget's layout manager may not have been set at that time.  We
            # also can't rely on the field's state because it may have been set
            # by Qt (e.g. in response to changing the state of a parent), so we
            # use the shadow attribute which reflects the programmer's intent.
            label.setEnabled(IView(obj)._enabled)

        return super().eventFilter(obj, event)
