# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QStackedWidget, QTabWidget
from PyQt4.QtTest import QTest

from ....automate import AutomationError, IAutomatedTabBar
from ....model import adapt, Adapter, notify_observers, observe
from ....ui import ITabBar

from ..utils import as_QWidget
from ..widget_adapter import WidgetAdapter


@adapt(QTabWidget, to=ITabBar)
class QTabWidgetITabBarAdapter(WidgetAdapter):
    """ An adapter to implement ITabBar for a QTabWidget. """

    def configure(self, properties):
        """ Configure the widget. """

        self._qt4_old_view = None

        self.adaptee.currentChanged.connect(self._current_changed)

        super().configure(properties)

    @ITabBar.current_page.getter
    def current_page(self):
        """ Invoked to get the current page. """

        widget = self.adaptee.currentWidget()

        if widget is None:
            return None

        for view in self.views:
            if as_QWidget(view.view) is widget:
                return view

        return None

    @current_page.setter
    def current_page(self, value):
        """ Invoked to set the current page. """

        for view in self.views:
            if view is value:
                self.adaptee.setCurrentWidget(as_QWidget(view.view))
                break

    @ITabBar.tabs_visible.getter
    def tabs_visible(self):
        """ Invoked to get the tabs's visibility. """

        return self.adaptee.tabBar().isVisible()

    @tabs_visible.setter
    def tabs_visible(self, value):
        """ Invoked to set the tabs's visibility. """

        self.adaptee.setDocumentMode(not value)
        self.adaptee.tabBar().setVisible(value)

    @observe('views')
    def __views_changed(self, change):
        """ Invoked when the list of views changes. """

        # Remove any old views.
        for view in change.old:
            view.remove()

        # Add any new views.
        # FIXME: Note that we don't synch with the order of the list.
        for view in change.new:
            view.add(self.adaptee)

    def _current_changed(self, idx):
        """ Invoked when the current page changes. """

        widget = self.adaptee.widget(idx)

        if widget is None:
            new_view = None
        else:
            for new_view in self.views:
                if as_QWidget(new_view.view) is widget:
                    break
            else:
                # This should never happen.
                return

            # Note that if we don't do this then (on OS/X at least) when a tab
            # is closed the newly exposed tab doesn't get the focus.
            focus_widget = widget.focusWidget()
            if focus_widget is not None:
                focus_widget.setFocus()

        old_view = self._qt4_old_view
        self._qt4_old_view = new_view

        notify_observers('current_page', self, new_view, old_view)


@adapt(QTabWidget, to=IAutomatedTabBar)
class QTabWidgetIAutomatedTabBarAdapter(Adapter):
    """ An adapter to implement IAutomatedTabBar for a QTabWidget. """

    def simulate_select(self, id, delay, index):
        """ Simulate the user selecting a tab page.

        :param id:
            is the (possibly scoped) identifier of the widget.
        :param delay:
            is the delay in milliseconds between simulated events.
        :param index:
            is the index of the tab page.
        """

        tab_widget = self.adaptee

        if index < 0 or index >= tab_widget.count():
            raise AutomationError(id, 'select', "invalid index")

        if delay >= 0:
            QTest.qWait(delay)

        # This is the easiest way to make the page current.
        tab_widget.setCurrentIndex(index)
