# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ....model import Any, implements, Model
from ....ui import ITabPage

from ..utils import as_QWidget


@implements(ITabPage)
class DipTabPage(Model):
    """ The DipTabPage class implements a dummy widget that implements the
    :class:`~dip.ui.ITabPage` interface.  It is needed because this toolkit
    implements a tab bar and the corresponding tab pages as a single widget.
    """

    # The containing QTabWidget.
    _qtabwidget = Any()

    def add(self, qtabwidget):
        """ Called to add the tab page to a QTabWidget. """

        # Allow for the page being added to this QTabWidget before being
        # removed from another one.
        if self._qtabwidget is not None:
            self.remove()

        self._qtabwidget = qtabwidget

        self._add_page(self.view)

    def remove(self):
        """ Called to remove the tab page from its QTabWidget. """

        self._remove_page(self.view)
        self._qtabwidget = None

    def configure(self, properties):
        """ Configure the view. """

        # There is nothing to do.

    def set_focus(self):
        """ Give the focus to the view. """

        if self.view is not None:
            self.view.set_focus()

    @ITabPage.enabled.getter
    def enabled(self):
        """ Invoked to get the enabled state. """

        idx = self._view_index(self.view)

        return self._qtabwidget.isTabEnabled(idx) if idx >= 0 else self._enabled

    @enabled.setter
    def enabled(self, value):
        """ Invoked to set the enabled state. """

        idx = self._view_index(self.view)

        if idx >= 0:
            self._qtabwidget.setTabEnabled(idx, value)

    @ITabPage.tool_tip.getter
    def tool_tip(self):
        """ Invoked to get the tool tip. """

        idx = self._view_index(self.view)

        return self._qtabwidget.tabToolTip(idx) if idx >= 0 else self._tool_tip

    @tool_tip.setter
    def tool_tip(self, value):
        """ Invoked to set the tool tip. """

        idx = self._view_index(self.view)

        if idx >= 0:
            self._qtabwidget.setTabToolTip(idx, value)

    @ITabPage.whats_this.getter
    def whats_this(self):
        """ Invoked to get the "What's This?" help. """

        idx = self._view_index(self.view)

        return self._qtabwidget.tabWhatsThis(idx) if idx >= 0 else self._whats_this

    @whats_this.setter
    def whats_this(self, value):
        """ Invoked to set the "What's This?" help. """

        idx = self._view_index(self.view)

        if idx >= 0:
            self._qtabwidget.setTabWhatsThis(idx, value)

    @ITabPage.title.getter
    def title(self):
        """ Invoked to get the page's title. """

        idx = self._view_index(self.view)

        return self._qtabwidget.tabText(idx) if idx >= 0 else self._title

    @title.setter
    def title(self, value):
        """ Invoked to set the page's title. """

        idx = self._view_index(self.view)

        if idx >= 0:
            self._qtabwidget.setTabText(idx, value)

    @ITabPage.view.setter
    def view(self, value):
        """ Invoked to set the view. """

        # Handle the trivial case.
        if self._qtabwidget is None:
            return

        # Remove any old view.
        idx = self._remove_page(self._view)

        # Add the new view.
        self._add_page(value, idx)

    @ITabPage.views.getter
    def views(self):
        """ Invoked to get the views. """

        view = self.view

        return () if view is None else (view, )

    @views.setter
    def views(self, value):
        """ Invoked to set the views. """

        if len(value) > 1:
            raise ValueError(
                    "{0} can only contain a single view".format(
                            type(self.adaptee).__name__))

        self.view = value[0] if len(value) != 0 else None

    def _add_page(self, view, idx=-1):
        """ Add a view at a particular position. """

        # Handle the trivial case.
        if view is None:
            return

        idx = self._qtabwidget.insertTab(idx, as_QWidget(view), self._title)

        # Configure the new tab.
        self._qtabwidget.setTabEnabled(idx, self._enabled)
        self._qtabwidget.setTabToolTip(idx, self._tool_tip)
        self._qtabwidget.setTabWhatsThis(idx, self._whats_this)

    def _remove_page(self, view):
        """ Remove a view and return its index. """

        idx = self._view_index(view)
        if idx >= 0:
            self._qtabwidget.removeTab(idx)

            # If we don't do this now then QApplication won't emit
            # focusChanged().
            as_QWidget(view).setParent(None)

        return idx

    def _view_index(self, view):
        """ Return the index of a view in the tab bar or -1 if there is no
        view, or no tab bar.
        """

        if self._qtabwidget is None or view is None:
            return -1

        return self._qtabwidget.indexOf(as_QWidget(view))
