# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Callable, MappingProxy, Model, unadapted
from ..toolkits import Toolkit

from .i_controller import IController
from .editor_factory import EditorFactory
from .i_view import IView
from .view_factory import ViewFactory


class Bindings(Model):
    """ The Bindings class represents set of bindings between widgets of a user
    interface and attributes of a model.
    """

    # The controller factory.  This must return an object that implements or
    # can be adapted to the :class:`~dip.ui.IController` interface.
    controller_factory = Callable()

    def __init__(self, **bindings):
        """ Initialise the bindings.

        :param \*\*bindings:
            the bindings specified as keyword arguments.  The argument name is
            the object name of the widget that the attribute will be bound to.
            The argument value is the binding.  If the argument value is a
            string then an :class:`~dip.ui.EditorFactory` instance will be
            automatically created for it.
        """

        self._bindings = bindings

    def bind(self, view, model, controller=None):
        """ Bind a view to a model and optional controller.

        :param view:
            is the view.
        :param model:
            is the model which is either a :class:`~dip.model.Model` instance
            or a Python mapping object.
        :param controller:
            is the optional controller.  If this is ``None`` then
            :attr:`~dip.ui.Bindings.controller_factory` will be called to
            create one.
        """

        # Make sure we have a Model for the model.
        if not isinstance(model, Model):
            model = MappingProxy(model)

        # Make sure we have a controller.
        if controller is None:
            # Pass the model rather than set it explicitly to allow the
            # controller to use observe() as a decorator.
            controller = self.controller_factory(model=model)

        controller = IController(controller)

        # Make sure we have the view as a toolkit object and its adapted form.
        toolkit_view = unadapted(view)
        if toolkit_view is view:
            view = _view_from_toolkit_view(toolkit_view)

        controller.view = view

        finalize = []

        for view_name, itm in self._bindings.items():
            # Find the sub-view in the user interface.
            toolkit_subview = Toolkit.find_toolkit_view(toolkit_view,
                    view_name)
            subview = _view_from_toolkit_view(toolkit_subview)

            # Automatically create a factory if needed.
            # FIXME: Do we really need a factory at all?
            if isinstance(itm, EditorFactory):
                name = itm.bind_to
                subview_factory = itm
            elif isinstance(itm, ViewFactory):
                name = None
                subview_factory = itm
            elif isinstance(itm, str):
                name = itm

                # Create a view factory for the subview.
                subview_factory_factory = _factory_for_view(subview)
                subview_factory = subview_factory_factory(itm, id=view_name)
            else:
                raise TypeError(
                        "the binding for '{0}' must be either an instance of "
                        "a ViewFactory sub-class or a string, not "
                        "'{1}'".format(view_name, type(itm).__name__))

            # Update the factory with the subview's name so that it doesn't get
            # changed when the toolkit subview is configured.
            subview_factory.id = view_name

            if name is None:
                subview_factory.bind_view(subview, controller)
            else:
                subview_factory.bind_editor(subview, model, name, controller)

            finalize.append((subview_factory, subview))

        for subview_factory, subview in finalize:
            subview_factory.finalize(subview)

    @controller_factory.default
    def controller_factory(self):
        """ Invoked to return the default controller factory. """

        from .controller import Controller

        return Controller


# FIXME: There should be a way for view factories to be registered so that we
#        can do this without hardcoding this map.
from .i_check_box import ICheckBox
from .check_box import CheckBox

from .i_combo_box import IComboBox
from .combo_box import ComboBox

from .i_dialog import IDialog
from .dialog import Dialog

from .i_dock import IDock
from .dock import Dock

from .i_filesystem_location_editor import IFilesystemLocationEditor
from .filesystem_location_editor import FilesystemLocationEditor

from .i_float_spin_box import IFloatSpinBox
from .float_spin_box import FloatSpinBox

from .i_form import IForm
from .form import Form

from .i_grid import IGrid
from .grid import Grid

from .i_group_box import IGroupBox
from .group_box import GroupBox

from .i_h_box import IHBox
from .h_box import HBox

from .i_label import ILabel
from .label import Label

from .i_line_editor import ILineEditor
from .line_editor import LineEditor

from .i_list_editor import IListEditor
from .list_editor import ListEditor

from .i_main_window import IMainWindow
from .main_window import MainWindow

from .i_menu import IMenu
from .menu import Menu

from .i_menu_bar import IMenuBar
from .menu_bar import MenuBar

from .i_message_area import IMessageArea
from .message_area import MessageArea

from .i_option_list import IOptionList
from .option_list import OptionList

from .i_push_button import IPushButton
from .push_button import PushButton

from .i_radio_buttons import IRadioButtons
from .radio_buttons import RadioButtons

from .i_spin_box import ISpinBox
from .spin_box import SpinBox

from .i_splitter import ISplitter
from .splitter import Splitter

from .i_storage_location_editor import IStorageLocationEditor
from .storage_location_editor import StorageLocationEditor

from .i_tab_bar import ITabBar
from .tab_bar import TabBar

from .i_table_editor import ITableEditor
from .table_editor import TableEditor

from .i_text_editor import ITextEditor
from .text_editor import TextEditor

from .i_tool_button import IToolButton
from .tool_button import ToolButton

from .i_v_box import IVBox
from .v_box import VBox

from .i_view_stack import IViewStack
from .view_stack import ViewStack

from .i_wizard import IWizard
from .wizard import Wizard

from .i_wizard_page import IWizardPage
from .wizard_page import WizardPage

_interface_factory_map = (
    (ICheckBox,                 CheckBox),
    (IComboBox,                 ComboBox),
    (IDialog,                   Dialog),
    (IDock,                     Dock),
    (IFilesystemLocationEditor, FilesystemLocationEditor),
    (IFloatSpinBox,             FloatSpinBox),
    (IForm,                     Form),
    (IGrid,                     Grid),
    (IGroupBox,                 GroupBox),
    (IHBox,                     HBox),
    (ILabel,                    Label),
    (ILineEditor,               LineEditor),
    (IListEditor,               ListEditor),
    (IMainWindow,               MainWindow),
    (IMenu,                     Menu),
    (IMenuBar,                  MenuBar),
    (IMessageArea,              MessageArea),
    (IOptionList,               OptionList),
    (IPushButton,               PushButton),
    (IRadioButtons,             RadioButtons),
    (ISpinBox,                  SpinBox),
    (ISplitter,                 Splitter),
    (IStorageLocationEditor,    StorageLocationEditor),
    (ITabBar,                   TabBar),
    (ITableEditor,              TableEditor),
    (ITextEditor,               TextEditor),
    (IToolButton,               ToolButton),
    (IVBox,                     VBox),
    (IViewStack,                ViewStack),
    (IWizard,                   Wizard),
    (IWizardPage,               WizardPage)
)


def _view_from_toolkit_view(toolkit_view):
    """ Return the view for a toolkit view. """

    # FIXME: This is probably toolkit specific because a particular toolkit
    #        view may be able to be adapted to more than one view interface and
    #        only the toolkit knows which would be the best choice.
    for interface, _ in _interface_factory_map:
        view = interface(toolkit_view, exception=False)
        if view is not None:
            return view

    return IView(toolkit_view)


def _factory_for_view(view):
    """ Return the factory for the factory that would create a view. """

    for interface, factory in _interface_factory_map:
        if isinstance(view, interface):
            return factory

    raise TypeError(
            "there is no editor factory that creates instances of {0}".format(
                    type(view).__name__))
