# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


import os.path

from ..model import Bool, Enum, implements, Model

from .i_validator import IValidator


@implements(IValidator)
class FilesystemLocationValidator(Model):
    """ The FilesystemLocationValidator class implements an editor validator
    for filesystem locations.
    """

    # The mode of the editor.  'open_file' means a single, existing file will
    # be obtained.  'save_file' means that a single file that may or may not
    # exist will be obtained.  'directory' means a single, existing directory
    # will be obtained.
    mode = Enum('open_file', 'save_file', 'directory')

    # This is set if a value is required.
    required = Bool(False)

    def configure(self, editor):
        """ Configure an editor to use this validator.

        :param editor:
            is the editor.
        """

    def validate(self, controller, editor, value):
        """ Validate an editor's value.

        :param controller:
            is the controller.
        :param editor:
            is the editor.
        :param value:
            is the value.
        :return:
            a string describing why the value is invalid.  An empty string is
            returned if the value is valid.
        """

        if value == '':
            ok = not self.required
        else:
            if self.mode == 'open_file':
                ok = os.path.isfile(value)
            elif self.mode == 'directory':
                ok = os.path.isdir(value)
            else:
                ok = True

        if ok:
            invalid_reason = ''
        elif self.mode == 'open_file':
            invalid_reason = "the name of an existing file is required"
        elif self.mode == 'save_file':
            invalid_reason = "the name of a file is required"
        elif self.mode == 'directory':
            invalid_reason = "the name of an existing directory is required"

        return invalid_reason
