# Copyright (c) 2011 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, Instance, Interface, List, Model, Str

from .i_action import IAction
from .i_editor import IEditor
from .i_message_area import IMessageArea
from .i_view import IView


class IController(Interface):
    """ The IController interface defines the API to be implemented by a
    :term:`view's<view>` :term:`controller`.
    """

    # The list of actions.
    actions = List(IAction)

    # This determines if the controller should automatically update the model
    # when the user changes the value of an editor.
    auto_update_model = Bool(True)

    # The list of editors being controlled.
    editors = List(IEditor)

    # This explains why the view is invalid.  It will be an empty string if
    # the view is valid.
    invalid_reason = Str()

    # The message areas for giving feedback to the user.
    message_areas = List(IMessageArea)

    # The model.
    model = Instance(Model)

    # The view.
    view = Instance(IView)

    def add_action(self, action):
        """ Add an action to the controller.

        :param action:
            is the action.
        """

    def add_editor(self, editor):
        """ Add an editor to the controller.

        :param editor:
            is the editor.
        """

    def update_model(self):
        """ Update the all the attributes of a model that have a corresponding
        editor.
        """

    def validate(self):
        """ Validate the view and update the explanation as to why it is (or
        isn't) invalid.
        """

    def validate_view(self):
        """ This is called after the view has been updated and only if all the
        view's editors contain valid values giving the controller an
        opportunity to carry out cross-editor validation.

        :return:
            a string which will be empty if the view is valid, otherwise it
            will explain why the view is invalid.
        """

    def update_view(self):
        """ This is called after the view has been validated giving the
        controller an opportunity to update the state of the view.
        """

    def is_valid(self, editor=None):
        """ Return the validity of an editor's (or all editors) current value.

        :param editor:
            is the editor.  If this is ``None`` then the validity of all
            editors is returned.
        :return:
            ``True`` if the editor (or all editors) is valid.
        """

    def find_action(self, id):
        """ Find the action with a particular identifier.

        :param id:
            is the identifier of the action.
        :return:
            the action or ``None`` if there was no such action.
        """

    def find_editor(self, id):
        """ Find the editor with a particular identifier.

        :param id:
            is the identifier of the editor.
        :return:
            the editor or ``None`` if there was no such editor.
        """

    def __getattr__(self, name):
        """ Reimplemented to support convenient access to actions and editors
        using a name derived from an identifier.
        
        :param name:
            the name of the attribute.  If this has a suffix of ``_action`` or
            ``_editor`` then the rest of the name is interpreted as the
            identifier of an action or editor (which is normally the name of
            the model attribute that the action or editor is bound to).
        :return:
            the attribute.  An AttributeError will be raised if there was no
            such attribute, action or editor.
        """
