# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Bool, Enum, Instance, List, Str

from .i_editor_binding import IEditorBinding
from .editor_factory import EditorFactory


class OptionSelectorFactory(EditorFactory):
    """ The OptionSelectorFactory class is a base class for editors that allow
    an option to be selected.
    """

    # This is set if ``None`` is allowed as a valid value.  If it is not
    # explicitly set then any corresponding value in the type's meta-data is
    # used.
    allow_none = Bool(None, allow_none=True)

    # The optional option labels.  These are used to visualise the
    # corresponding options.
    labels = List(Str())

    # The options.  If it is a string then it is the name of the attribute in
    # the model where the options can be found.  If an option has no
    # corresponding label then it will be adapted to the
    # :class:`~dip.ui.IDisplay` interface and its :attr:`~dip.ui.IDisplay.name`
    # used (which defaults to its string representation).
    options = Instance(Str(), List())

    def configure_with_attribute_type(self, view, attribute_type):
        """ Configure the view possibly using values from the bound attribute's
        type.

        :param view:
            is the view to configure.
        :param attribute_type:
            is the attribute's type.
        """

        if isinstance(attribute_type, Enum):
            options = attribute_type.members
            labels = [self.get_natural_name(option) for option in options]
        else:
            options = self.options
            labels = self.labels

            if isinstance(options, str):
                options = getattr(IEditorBinding(view).model, options)

        view.labels = labels
        view.options = () if options is None else options

        super().configure_with_attribute_type(view, attribute_type)

    def set_default_validator(self, view):
        """ Sets a view's default validator.

        :param view:
            is the view.
        """

        # See if a value is required.
        allow_none = self.allow_none
        if allow_none is None:
            attribute_type = IEditorBinding(view).attribute_type
            if attribute_type is not None:
                allow_none = attribute_type.allow_none
            else:
                allow_none = False

        # See if a validator is required.
        if not allow_none:
            from .option_validator import OptionValidator

            view.validator = OptionValidator()
