# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ..model import Callable, MappingProxy, Model, Str
from ..toolkits import Toolkit

from .i_controller import IController
from .i_view_binding import IViewBinding


class ViewFactory(Model):
    """ The ViewFactory class is the base class for all factories that
    create views, sub-views and editors.
    """

    # The controller factory.  This must return an object that implements or
    # can be adapted to the :class:`~dip.ui.IController` interface.
    controller_factory = Callable()

    # The view's identifier.  If it is not specified then a sub-class may
    # implements an appropriate default.
    id = Str()

    # The name of the toolkit factory method.
    toolkit_factory = Str()

    def __init__(self, **properties):
        """ Initialize the factory.

        :param \*\*properties:
            are keyword arguments used as property names and values that are
            applied to the each object created by the factory.
        """

        self._properties = properties

    def __call__(self, model=None, controller=None, parent=None, top_level=True, **properties):
        """ Create a view and bind it to a model and optional controller.

        :param model:
            is the optional model which is either a :class:`~dip.model.Model`
            instance or a Python mapping object.
        :param controller:
            is the optional controller.  If this is ``None`` then
            :attr:`~dip.ui.ViewFactory.controller_factory` will be called to
            create one.
        :param parent:
            is the optional parent view.
        :param top_level:
            is ``True`` if the view is to be displayed as a top-level widget.
        :param \*\*properties:
            are any remaining keyword arguments and are interpreted as
            attribute names and values to be set for the view.
        :return:
            the view.
        """

        # Make sure we have a Model for the model.
        if model is None:
            model = Model()
        elif not isinstance(model, Model):
            model = MappingProxy(model)

        # Make sure we have a controller.
        if controller is None:
            # Pass the model rather than set it explicitly to allow the
            # controller to use observe() as a decorator.
            # FIXME: This only works if the object returned by the factory
            #        implements IController rather than just being able to be
            #        adapted to it.
            controller = IController(self.controller_factory(model=model))
        else:
            controller = IController(controller)
            controller.model = model

        # Create the view itself.
        view = self.create(model, controller, parent)

        controller.view = view

        for name, value in properties.items():
            setattr(view, name, value)

        # Allow further configuration now that everything is created.
        self.finalize(view)

        # Update the view according to the current state of the model.
        controller.validate()
        controller.update_view()

        if top_level:
            # Make sure we can display the view.
            view = Toolkit.displayable(view)

        return view

    def create(self, model, controller, parent):
        """ Create the view instance.

        :param model:
            is the model.
        :param controller:
            is the controller.
        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

        view = self.create_view(parent)
        self.bind_view(view, controller)

        return view

    def create_view(self, parent):
        """ Use the toolkit to create an instance of the view.

        :param parent:
            is the optional parent view.
        :return:
            the view.
        """

        return getattr(Toolkit, self.toolkit_factory)(parent)

    def bind_view(self, view, controller):
        """ Bind a view created by this factory.

        :param view:
            is the view.
        :param controller:
            is the controller.
        """

        iviewbinding = IViewBinding(view)
        iviewbinding.controller = controller
        iviewbinding.factory = self

        self.configure(view)

    def configure(self, view):
        """ Reimplemented to configure the IView specific attributes.

        :param view:
            is the view to configure.  It should be an implementation of the
            :class:`~dip.ui.IView` interface.
        """

        view.id = self.id

        # First configure any attributes that defined by the view interface.
        # Any remaining attributes are assumed to be toolkit specific.
        toolkit_properties = {}

        for name, value in self._properties.items():
            if hasattr(view, name):
                setattr(view, name, value)
            else:
                toolkit_properties[name] = value

        view.configure(toolkit_properties)

    def finalize(self, view):
        """ This is called when the creation of a view has been completed.
        This default implementation does nothing.

        :param view:
            is the view created by this factory.
        """

    @staticmethod
    def get_natural_name(name):
        """ Convert a name, typically of an attribute, to a more natural name.

        :param name:
            is the original name.
        :return:
            the more natural version.
        """

        name = name.replace('_', ' ')
        name = name.strip()
        name = name[0].upper() + name[1:]

        return name

    @controller_factory.default
    def controller_factory(self):
        """ Invoked to return the default controller factory. """

        from .controller import Controller

        return Controller
