# Copyright (c) 2010 Riverbank Computing Limited.


import ast
import inspect
import io
import sys
import tokenize


class ModuleParser:
    """ The ModuleParser class parses a module extracting all of the
    documentation for its public API.  This can then be rendered as reST.
    """

    def __init__(self):
        """ Initialise the parser. """

        self.warnings = []

    def parse(self, module_name):
        """ Parse a module.

        :param module_name:
            is the name of the module.
        """

        self._rst = []

        self._module_name = module_name

        # Import the module and introspect it.
        try:
            __import__(self._module_name, None, None)
        except ImportError:
            self.warnings.append(
                    "couldn't import the '{0}' module".format(
                            self._module_name))
            mod = None
        else:
            mod = sys.modules.get(self._module_name)

        if mod is None:
            return

        self._introspect_module(mod)

    def _introspect_module(self, mod):
        """ Introspect a module. """

        self._heading(':mod:`{0}`'.format(self._module_name), '=')
        self._text('.. module:: {0}'.format(self._module_name))

        # Get the module's optional doc string.
        self._doc_string(mod)

        for obj_name, obj in inspect.getmembers(mod):
            if obj_name.startswith('_'):
                continue

            if inspect.isclass(obj):
                if issubclass(obj, Exception):
                    self._introspect_exception(obj_name, obj)
                else:
                    self._introspect_class(obj_name, obj)
            elif inspect.isfunction(obj):
                self._introspect_function(obj_name, obj)
            elif inspect.ismodule(obj):
                pass
            else:
                self.warnings.append(
                        "'{0}.{1}' is neither a class or a function".format(
                                self._module_name, obj_name))

    def _introspect_class(self, name, klass):
        """ Introspect a class. """

        # Add the class name heading.
        self._text()
        self._text()
        self._heading(':class:`{0}`'.format(name))
        self._text('.. class:: {0}'.format(name))

        # Add any base classes.
        if len(klass.__bases__) > 1:
            self._class_references('Base classes', klass.__bases__)
        elif klass.__bases__[0] is not object:
            self._class_references('Base class', klass.__bases__)

        # Add any docstring.
        self._doc_string(klass, indent=1)

        # Add the class's API.
        attrs, methods = self._get_class_api(klass)

        for attr in attrs:
            self._text()
            self._text('.. attribute:: {0}'.format(attr), indent=1)

            if attr.comments:
                self._text()

                for comment in attr.comments:
                    self._text(comment, indent=2)

        for method in methods:
            self._text()
            self._text('.. {0}:: {1}'.format(method.directive(), method),
                    indent=1)
            self._doc_string(method.function(), indent=2)

    def _get_class_api(self, klass):
        """ Get the lists of AttrAPI and MethodAPI instances corresponding to
        the typed attributes and methods of a class.
        """

        attrs = []
        methods = []

        # Go through the source code looking for typed attributes because
        # inspect.getcomments() doesn't support class attributes.
        klass_source = inspect.getsource(klass)
        klass_comments = ClassComments(klass_source)

        try:
            module_ast = ast.parse(klass_source)
        except SyntaxError:
            self.warnings.append(
                    "the source code for {0} couldn't be compiled".format(
                            klass.__name__))

            return [], []

        klass_ast = next(ast.iter_child_nodes(module_ast))

        for node in ast.iter_child_nodes(klass_ast):
            if isinstance(node, ast.Assign):
                attr = AttrAPI.parse(node, klass)
                if attr:
                    attr.comments = klass_comments[node.lineno]
                    attrs.append(attr)

        # Get the class's objects and sort their names.
        api = vars(klass)
        names = sorted(api.keys())

        # We want __init__ to be the first method if it is present.
        if '__init__' in names:
            names.remove('__init__')
            names.insert(0, '__init__')

        for obj_name in names:
            obj = api[obj_name]

            # Remove private methods but keep special methods.
            if obj_name.startswith('_'):
                if obj_name.startswith('__') and obj_name.endswith('__'):
                    pass
                else:
                    continue

            if inspect.isfunction(obj) or isinstance(obj, (staticmethod, classmethod)):
                methods.append(MethodAPI(obj_name, obj, klass))

        return attrs, methods

    def _introspect_exception(self, name, exception):
        """ Introspect an exception. """

        # Add the exception name heading.
        self._text()
        self._text()
        self._heading(':exc:`{0}`'.format(name))
        self._text('.. exception:: {0}'.format(name))

        # Add any base exceptions.
        self._class_references('Base exception', exception.__bases__)

        # Add any docstring.
        self._doc_string(exception, indent=1)

    def _introspect_function(self, name, function):
        """ Introspect a function. """

        argspec = inspect.getfullargspec(function)
        args = inspect.formatargspec(*argspec)

        self._text()
        self._text()
        self._heading(':func:`{0}`'.format(name))
        self._text('.. function:: {0}{1}'.format(name, args))
        self._doc_string(function, indent=1)

    def to_string(self):
        """ Convert the parsed documentation to a string.

        :return:
            the text as a string.
        """

        return '\n'.join(self._rst) + '\n'

    def _class_references(self, context, klasses):
        """ Add a list of class references in a particular context. """

        if klasses:
            references = [self.class_reference(k) for k in klasses]

            self._text()
            self._text('{0}: {1}'.format(context, ', '.join(references)),
                    indent=1)

    @staticmethod
    def class_reference(klass):
        """ Return a reST reference to the given class. """

        return ':class:`~{0}`'.format(ModuleParser._preferred_name(klass))

    @staticmethod
    def _preferred_name(obj):
        """ Return the prefered reference to the object (which must have
        __module__ and __name__ attributes).
        """

        if obj.__module__ == '__builtin__':
            return obj.__name__

        # The preferred name is the grandparent module, not the parent one.
        obj_module_path = obj.__module__.split('.')

        while len(obj_module_path) > 1:
            obj_module_path.pop()
            module_name = '.'.join(obj_module_path)

            try:
                __import__(module_name)
            except:
                mod = None
            else:
                mod = sys.modules[module_name]

            if mod and obj.__name__ and getattr(mod, obj.__name__, None) is obj:
                break
        else:
            # We couldn't find anything so use the module the object was
            # defined in.
            module_name = obj.__module__

        return '{0}.{1}'.format(module_name, obj.__name__)

    def _text(self, text="", indent=0):
        """ Render some (optional) text. """

        self._rst.append('    ' * indent + text)

    def _heading(self, heading, heading_style='-'):
        """ Render a heading. """

        self._text(heading)
        self._text(heading_style * len(heading))

    def _doc_string(self, obj, indent=0):
        """ Render an object's doc string. """

        doc_string = inspect.getdoc(obj)
        if doc_string:
            self._text()

            for line in doc_string.splitlines():
                self._text(line, indent=indent)


class ClassComments:
    """ The ClassComments class encapsulates the comments in the source code
    of a class.
    """

    def __init__(self, klass_source):
        """ Initialise the object. """

        sio = io.StringIO(klass_source)
        blocks = []
        block = None
        last_code_line = -1

        for token, comment, start, end, line in tokenize.tokenize(lambda: sio.readline().encode('utf8')):
            if token == tokenize.COMMENT:
                # To keep it simple we only use the first line if the comment
                # is extended over multiple lines using line continuation, but
                # we still remember that it spanned several lines.
                comment = comment.splitlines()[0].rstrip()

                # Work out the position of the first significant character of
                # the comment (ie. after all whitespace and comment
                # characters).
                original_length = len(comment)

                while comment and comment.startswith('#'):
                    comment = comment[1:].lstrip()

                comment_offset = original_length - len(comment)

                # Get the start and end lines and the start column.
                start_line, start_column = start
                end_line, _ = end

                start_column += comment_offset

                # Create a new block if there isn't one, or the new comment
                # isn't contiguous.
                if block is None or not block.add_comment(comment, start_line, start_column, end_line):
                    block = CommentBlock(comment, start_line, start_column, end_line)
                    blocks.append(block)

                if start_line == last_code_line:
                    # The comment is on the same line as a non-comment so
                    # assume it is a line comment refering to the previous
                    # token rather than a block comment refering to the
                    # following token.
                    block.line_comment = True
            elif token != tokenize.NL:
                block = None
                last_code_line = line

        # Convert each block to a list of properly indented strings and
        # associate them with a particular line.
        self._comments = {}

        for block in blocks:
            if block.line_comment:
                lineno = block.block_start
            else:
                lineno = block.block_end + 1

            comment_lines = []

            for comment, start_column in block.comments:
                # Work out the indent needed by this line to maintain its
                # position relative to the other lines in the block.
                indent = ' ' * (start_column - block.block_start_column)

                comment_lines.append(indent + comment)

            self._comments[lineno] = comment_lines

    def __getitem__(self, lineno):
        """ Get the comments related to the given line as a list of strings.
        Indentation within the comments is preserved.
        """

        return self._comments.get(lineno, [])


class CommentBlock:
    """ The CommentBlock class encapsulates a block of comments, ie. a set of
    contiguous comment lines.
    """

    def __init__(self, comment, start_line, start_column, end_line):
        """ Initialise the object. """

        # Each comment is a tuple of the stripped comment text and the position
        # of the first character in it's original context.
        self.comments = [(comment, start_column)]

        # The line containing the first comment of the block.
        self.block_start = start_line

        # The start column of the block.
        self.block_start_column = start_column

        # The line containing the last comment of the block.
        self.block_end = end_line

        # Set if the comment is a line comment, ie. refers to a non-comment
        # earlier in the same line.
        self.line_comment = False

    def add_comment(self, comment, start_line, start_column, end_line):
        """ Adds a new comment line to the block if possible, otherwise return
        ``False`` if it is part of a new block.
        """

        # It can't be added to the block if it isn't contiguous.
        if start_line > self.block_end + 1:
            return False

        # Extend the block downwards.
        self.comments.append((comment, start_column))
        self.block_end = end_line

        # Extend the block leftwards if necessary.
        if self.block_start_column > start_column:
            self.block_start_column = start_column

        return True


class MethodAPI:
    """ The MethodAPI class encapsulates a single method of a class. """

    def __init__(self, name, method, klass):
        """ Initialise the object. """

        # The method name.
        self.name = name

        # The method object.
        self._method = method

        # The name of the containing class.
        self._klass_name = klass.__name__

    def directive(self):
        """ Return the appropriate Sphinx directive for the method. """

        if isinstance(self._method, staticmethod):
            return 'staticmethod'

        if isinstance(self._method, classmethod):
            return 'classmethod'

        return 'method'

    def function(self):
        """ Return the function object. """

        if isinstance(self._method, (staticmethod, classmethod)):
            return self._method.__get__(self._method)

        return self._method

    def __str__(self):
        """ Return a string representation of the method. """

        argspec = inspect.getfullargspec(self.function())

        # Remove any 'self' or 'cls'.
        if argspec[0] and argspec[0][0] in ('self', 'cls'):
            del argspec[0][0]

        args = inspect.formatargspec(*argspec)

        return '{0}.{1}{2}'.format(self._klass_name, self.name, args)


class AttrAPI(object):
    """ The AttrAPI class encapsulates an individual typed attribute. """

    def __init__(self, name, definition):
        """ Initialise the object. """

        # The attribute name.
        self.name = name

        # The attribute definition.
        self._definition = definition

        # The attribute comments.
        self.comments = []

    @staticmethod
    def parse(node, klass):
        """ Parse the given AST node and return a AttrAPI instance if it was a
        typed attribute or ``None`` if it wasn't.
        """

        name = next(ast.iter_child_nodes(node)).id
        if name.startswith('_'):
            return None

        definition = Decompiler.decompile(node.value, klass)
        if not definition:
            return None

        return AttrAPI(name, definition)

    def __str__(self):
        """ Return a string representation of the attribute. """

        return '{0} = {1}'.format(self.name, self._definition)

    def __eq__(self, other):
        """ The attribute name is used for comparison purposes. """

        return self.name == other


class Decompiler:
    """ A simple AST decompiler that will decompile a typed attribute
    definition.
    """

    def __init__(self, klass):
        """ Initialise the object. """

        # The module containing the definition.
        self._module = sys.modules[klass.__module__]

    @staticmethod
    def decompile(node, klass):
        """ Decompile the given node, assuming it is a typed attribute
        definition, and return the source code or ``None`` if it wasn't a typed
        attribute.
        """

        try:
            source = Decompiler(klass)._decompile_node(node)
        except ValueError:
            source = None

        return source

    def _decompile_node(self, node):
        """ Decompile the given node. """

        method = getattr(self, '_' + node.__class__.__name__)

        return method(node)

    def _cross_reference_name(self, name):
        """ Return a cross-reference to the given class name. """

        xref = None

        obj = getattr(self._module, name, None)

        if inspect.isclass(obj):
            xref = ModuleParser.class_reference(obj)

        return xref

    def _cross_reference_path(self, path):
        """ Return a cross-reference to the given class path name. """

        xref = None

        path_list = path.split('.')

        if len(path_list) > 1:
            module_name = '.'.join(path_list[:-1])

            try:
                __import__(module_name)
                module = sys.modules[module_name]
            except:
                module = None

            if module:
                obj = getattr(module, path_list[-1], None)

                if inspect.isclass(obj):
                    xref = ModuleParser.class_reference(obj)

        return xref

    def _Call(self, node):
        """ A function call (probably a typed attribute factory). """

        name = self._decompile_node(node.func)

        # Replace an instance type with a cross-reference if possible.
        inst_arg = None

        if name in ('Instance', 'List', 'Subclass', 'Type') and len(node.args) > 0:
            inst = node.args[0]

            if isinstance(inst, ast.Name):
                # For the moment at least, Sphinx doesn't resolve links in
                # attribute values.
                #inst_arg = self._cross_reference_name(inst.id)
                inst_arg = inst.id
            elif isinstance(inst, ast.Str):
                # For the moment at least, Sphinx doesn't resolve links in
                # attribute values.
                #inst_arg = self._cross_reference_path(inst.s)
                inst_arg = inst.s

        if inst_arg:
            args = [self._decompile_node(arg) for arg in node.args[1:]]
            args.insert(0, inst_arg)
        else:
            args = [self._decompile_node(arg) for arg in node.args]

        args.extend([self._decompile_node(arg) for arg in node.keywords])

        return '{0}({1})'.format(name, ', '.join(args))

    def _Num(self, node):
        """ A number. """

        return str(node.n)

    def _Str(self, node):
        """ A string. """

        return '\'' + node.s.replace('\n', '\\n') + '\''

    def _Attribute(self, node):
        """ An attribute get. """

        return '{0}.{1}'.format(self._decompile_node(node.value), node.attr)

    def _keyword(self, node):
        """ A keyword. """

        return '{0}={1}'.format(node.arg, self._decompile_node(node.value))

    def _List(self, node):
        """ A list. """

        elems = [self._decompile_node(elem) for elem in node.elts]

        return '[' + ', '.join(elems) + ']'

    def _Name(self, node):
        """ A name. """

        return node.id

    def _Tuple(self, node):
        """ A tuple. """

        elems = [self._decompile_node(elem) for elem in node.elts]

        return '(' + ', '.join(elems) + ')'

    def _UnaryOp(self, node):
        """ A unary operator. """

        return self._decompile_node(node.op) + self._decompile_node(node.operand)

    def _USub(self, node):
        """ Unary minus. """

        return '-'


if __name__ == '__main__':
    # Create the command line parser.
    import optparse

    clp = optparse.OptionParser(usage="python %prog [opts] module")

    clp.add_option("-o", "--output-file", type='string', metavar="FILE",
            dest='outfile',
            help="the name of the output file [default: stdout]")

    opts, args = clp.parse_args()

    if len(args) != 1:
        clp.print_help()
        sys.exit(2)

    module = args[0]

    if opts.outfile:
        out_file = open(opts.outfile, 'wt')
    else:
        out_file = sys.stdout

    # Parse and render the module.
    parser = ModuleParser()
    parser.parse(module)

    if parser.warnings:
        sys.stderr.write('\n'.join(parser.warnings) + '\n')
        #sys.exit(1)

    out_file.write(parser.to_string())
