# Copyright (c) 2010 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from dip.model import Model, observe, Set, Str, ValidationError


class Klass(Model):
    """ A class for test models. """

    default_set = Set()

    explicit_default_set = Set(default={None})

    attr = Set(Str())

    def __init__(self):
        super().__init__()

        self.change = None
        self.changes = 0
        observe('attr', self, self.__observed)

    def __observed(self, change):
        self.change = change
        self.changes += 1


class TestSetType(unittest.TestCase):
    """ This tests the Set type. """

    def check_observed(self, model, new, old):
        """ Check that a test model observed a change. """

        self.assertEqual(model.changes, 1)

        change = model.change

        self.assertIs(change.model, model)
        self.assertEqual(change.name, 'attr')
        self.assertEqual(change.new, new)
        self.assertEqual(change.old, old)

    def check_unobserved(self, model):
        """ Check that a test model hasn't observed a change. """

        self.assertEqual(model.changes, 0)

    def test_invalid_set_type(self):
        """ Invalid Set type. """

        class BadSet(Model):
            bad = Set(0)

        with self.assertRaises(TypeError):
            BadSet(bad=set())

    def test_default_set_type_explicit_default(self):
        """ Default Set explicit default value. """

        k = Klass()
        self.assertSetEqual(k.explicit_default_set, {None})

    def test_set_subclass_default(self):
        """ Default Set sub-class. """

        class SubKlass(Klass):

            default_set = {1}

        k = SubKlass()
        self.assertSetEqual(k.default_set, {1})

    def test_default_set_type_default_value(self):
        """ Default Set type default value. """

        k = Klass()
        self.assertSetEqual(k.default_set, set())

    def test_default_set_type_valid_value(self):
        """ Default Set type valid value. """

        k = Klass()
        k.default_set = {1, True, 'one'}
        self.assertSetEqual(k.default_set, {1, True, 'one'})

    def test_default_set_type_invalid_value(self):
        """ Default Set type invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.default_set = None

    def test_set_type_valid_default_value(self):
        """ Set type valid default value. """

        k = Klass(attr={'abc', 'def'})
        self.assertSetEqual(k.attr, {'abc', 'def'})

    def test_set_type_invalid_default_value(self):
        """ Set type invalid default value. """

        with self.assertRaises(ValidationError):
            Klass(attr={False})

    def test_set_type_rebind_valid_value(self):
        """ Set type rebind valid value. """

        k = Klass()
        k.attr = {'abc', 'def'}
        self.assertSetEqual(k.attr, {'abc', 'def'})
        self.check_observed(k, {'abc', 'def'}, set())

    def test_set_type_rebind_unchanged_value(self):
        """ Set type rebind unchanged value. """

        k = Klass()
        k.attr = set()
        self.assertSetEqual(k.attr, set())
        self.check_unobserved(k)

    def test_set_type_rebind_invalid_value(self):
        """ Set type rebind invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr = {False}

    def test_set_type_rebind_invalid_set(self):
        """ Set type rebind invalid set. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr = k.default_set

    def test_set_add_valid(self):
        """ Set add() valid value. """

        k = Klass()
        k.attr.add('abc')
        self.assertSetEqual(k.attr, {'abc'})
        self.check_observed(k, {'abc'}, set())

    def test_set_add_invalid(self):
        """ Set add() invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr.add(False)

    def test_set_clear(self):
        """ Set clear(). """

        k = Klass(attr={'abc'})
        k.attr.clear()
        self.assertSetEqual(k.attr, set())
        self.check_observed(k, set(), {'abc'})

    def test_set_clear_unchanged(self):
        """ Unchanged set clear(). """

        k = Klass()
        k.attr.clear()
        self.assertSetEqual(k.attr, set())
        self.check_unobserved(k)

    def test_set_difference_update_valid(self):
        """ Set difference_update() valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr.difference_update({'abc'})
        self.assertSetEqual(k.attr, {'def'})
        self.check_observed(k, set(), {'abc'})

    def test_set_difference_update_unchanged(self):
        """ Unchanged set difference_update() valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr.difference_update({'xyz'})
        self.assertSetEqual(k.attr, {'abc', 'def'})
        self.check_unobserved(k)

    def test_set_difference_update_invalid(self):
        """ Set difference_update() invalid value. """

        k = Klass(attr={'abc', 'def'})
        with self.assertRaises(ValidationError):
            k.attr.difference_update({False})

    def test_set_isub_valid(self):
        """ Set -= valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr -= {'abc'}
        self.assertSetEqual(k.attr, {'def'})
        self.check_observed(k, set(), {'abc'})

    def test_set_isub_unchanged(self):
        """ Unchanged set -= valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr -= {'xyz'}
        self.assertSetEqual(k.attr, {'abc', 'def'})
        self.check_unobserved(k)

    def test_set_isub_invalid(self):
        """ Set -= invalid value. """

        k = Klass(attr={'abc', 'def'})
        with self.assertRaises(ValidationError):
            k.attr -= {False}

    def test_set_discard_valid(self):
        """ Set discard() valid value. """

        k = Klass(attr={'abc'})
        k.attr.discard('abc')
        self.assertSetEqual(k.attr, set())
        self.check_observed(k, set(), {'abc'})

    def test_set_discard_unchanged(self):
        """ Unchanged set discard() valid value. """

        k = Klass()
        k.attr.discard('abc')
        self.assertSetEqual(k.attr, set())
        self.check_unobserved(k)

    def test_set_discard_invalid(self):
        """ Set discard() invalid value. """

        k = Klass(attr={'abc'})
        with self.assertRaises(ValidationError):
            k.attr.discard(None)

    def test_set_intersection_update_valid(self):
        """ Set intersection_update() valid value. """

        k = Klass(attr={'abc', 'xyz'})
        k.attr.intersection_update({'abc', 'def'})
        self.assertSetEqual(k.attr, {'abc'})
        self.check_observed(k, set(), {'xyz'})

    def test_set_intersection_update_unchanged(self):
        """ Unchanged set intersection_update() valid value. """

        k = Klass(attr={'abc'})
        k.attr.intersection_update({'abc', 'def'})
        self.assertSetEqual(k.attr, {'abc'})
        self.check_unobserved(k)

    def test_set_intersection_update_invalid(self):
        """ Set intersection_update() invalid value. """

        k = Klass(attr={'abc', 'xyz'})
        with self.assertRaises(ValidationError):
            k.attr.intersection_update({False})

    def test_set_iand_valid(self):
        """ Set &= valid value. """

        k = Klass(attr={'abc', 'xyz'})
        k.attr &= {'abc', 'def'}
        self.assertSetEqual(k.attr, {'abc'})
        self.check_observed(k, set(), {'xyz'})

    def test_set_iand_unchanged(self):
        """ Unchanged set &= valid value. """

        k = Klass(attr={'abc'})
        k.attr &= {'abc', 'def'}
        self.assertSetEqual(k.attr, {'abc'})
        self.check_unobserved(k)

    def test_set_iand_invalid(self):
        """ Set &= invalid value. """

        k = Klass(attr={'abc', 'xyz'})
        with self.assertRaises(ValidationError):
            k.attr &= {False}

    def test_set_pop(self):
        """ Set pop(). """

        k = Klass(attr={'abc'})
        element = k.attr.pop()
        self.assertEqual(element, 'abc')
        self.assertSetEqual(k.attr, set())
        self.check_observed(k, set(), {'abc'})

    def test_set_remove_valid(self):
        """ Set remove() valid value. """

        k = Klass(attr={'abc'})
        k.attr.remove('abc')
        self.assertSetEqual(k.attr, set())
        self.check_observed(k, set(), {'abc'})

    def test_set_remove_invalid(self):
        """ Set remove() invalid value. """

        k = Klass(attr={'abc'})
        with self.assertRaises(ValidationError):
            k.attr.remove(None)

    def test_set_symmetric_difference_update_valid(self):
        """ Set symmetric_difference_update() valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr.symmetric_difference_update({'abc', 'ghi'})
        self.assertSetEqual(k.attr, {'def', 'ghi'})
        self.check_observed(k, {'ghi'}, {'abc'})

    def test_set_symmetric_difference_update_invalid(self):
        """ Set symmetric_difference_update() invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr.symmetric_difference_update({False})

    def test_set_ixor_valid(self):
        """ Set ^= valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr ^= {'abc', 'ghi'}
        self.assertSetEqual(k.attr, {'def', 'ghi'})
        self.check_observed(k, {'ghi'}, {'abc'})

    def test_set_ixor_invalid(self):
        """ Set ^= invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr ^= {False}

    def test_set_update_valid(self):
        """ Set update() valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr.update({'abc', 'ghi'})
        self.assertSetEqual(k.attr, {'abc', 'def', 'ghi'})
        self.check_observed(k, {'ghi'}, set())

    def test_set_update_unchanged(self):
        """ Unchanged set update() valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr.update({'abc', 'def'})
        self.assertSetEqual(k.attr, {'abc', 'def'})
        self.check_unobserved(k)

    def test_set_update_invalid(self):
        """ Set update() invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr.update({False})

    def test_set_ior_valid(self):
        """ Set |= valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr |= {'abc', 'ghi'}
        self.assertSetEqual(k.attr, {'abc', 'def', 'ghi'})
        self.check_observed(k, {'ghi'}, set())

    def test_set_ior_unchanged(self):
        """ Unchanged set |= valid value. """

        k = Klass(attr={'abc', 'def'})
        k.attr |= {'abc', 'def'}
        self.assertSetEqual(k.attr, {'abc', 'def'})
        self.check_unobserved(k)

    def test_set_ior_invalid(self):
        """ Set |= invalid value. """

        k = Klass()
        with self.assertRaises(ValidationError):
            k.attr |= {False}


if __name__ == '__main__':
    unittest.main()
