# Copyright (c) 2012 Riverbank Computing Limited.


try:
    import unittest2 as unittest
except ImportError:
    import unittest

from PyQt4.QtGui import QApplication
from PyQt4.QtTest import QTest

from dip.automate import AutomationError, Robot
from dip.model import Int, List, Model, Str, unadapted
from dip.ui import ListEditor, VBox


if __name__ == '__main__':
    import sys

    app = QApplication(sys.argv)


# The test model.
class TestRecord(Model):
    name = Str()

    age = Int()

class TestModel(Model):
    attr = List(TestRecord)

model = None


# The test GUI.
view_factory = VBox(ListEditor('attr'), id='ListEditorModel')

view = None


class TestListEditorWithModel(unittest.TestCase):
    """ This tests the list editor when an element is a model. """

    def setUp(self):
        """ Initialise the list widget, creating it if necessary. """

        # Create it if this is the first call.
        global model, view

        if model is None:
            model = TestModel()

        if view is None:
            view = view_factory(model=model)

        model.attr = []

        view.visible = True
        QTest.qWaitForWindowShown(unadapted(view))
        QApplication.setActiveWindow(unadapted(view))

    def tearDown(self):
        """ Tidy up the list widget. """

        view.visible = False

    def test_update(self):
        """ Test update. """

        model.attr = [TestRecord(name="Old", age=20)]
        Robot.simulate('ListEditorModel:attr', 'update', "New", 0, 'name')
        self.assertEqual(model.attr[0].name, "New")

    def test_update_invalid_index(self):
        """ Test update with an invalid index. """

        with self.assertRaises(AutomationError):
            Robot.simulate('ListEditorModel:attr', 'update', "Invalid", -1,
                    'name')

    def test_update_invalid_column(self):
        """ Test update with an invalid column specified. """

        with self.assertRaises(AutomationError):
            Robot.simulate('ListEditorModel:attr', 'update', "Invalid", 0,
                    'invalid')

    def test_update_no_column(self):
        """ Test update with no column specified. """

        with self.assertRaises(AutomationError):
            Robot.simulate('ListEditorModel:attr', 'update', "Invalid", 0)

    def test_append(self):
        """ Test append. """

        Robot.simulate('ListEditorModel:attr', 'append',
                TestRecord(name="New", age=40))
        self.assertEqual(len(model.attr), 1)
        self.assertEqual(model.attr[0].name, "New")
        self.assertEqual(model.attr[0].age, 40)

    def test_remove(self):
        """ Test remove. """

        model.attr = [TestRecord(name="First")]
        Robot.simulate('ListEditorModel:attr', 'remove', 0)
        self.assertEqual(len(model.attr), 0)

    def test_move(self):
        """ Test move. """

        model.attr = [TestRecord(name="First"), TestRecord(name="Second"),
                TestRecord(name="Third")]
        Robot.simulate('ListEditorModel:attr', 'move', 0, 2)
        self.assertEqual(model.attr[0].name, "Second")
        self.assertEqual(model.attr[1].name, "Third")
        self.assertEqual(model.attr[2].name, "First")


if __name__ == '__main__':
    unittest.main()
